package com.chelvc.cloud.vehicle.server.service.impl;

import com.alibaba.nacos.shaded.com.google.gson.Gson;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.constant.PlatformConstant;
import com.chelvc.cloud.vehicle.client.constant.TradeConstant;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.client.param.WithdrawParam;
import com.chelvc.cloud.vehicle.server.dao.AssetMapper;
import com.chelvc.cloud.vehicle.server.entity.Asset;
import com.chelvc.cloud.vehicle.server.entity.MerchantRelation;
import com.chelvc.cloud.vehicle.server.handle.TradeHandle;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.HashMap;

@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired,@Lazy}))
public class AssetServiceImpl extends ServiceImpl<AssetMapper, Asset> implements AssetService {
    private final BalanceDetailService balanceDetailService;
    private final OmsOrderService orderService;
    private final MerchantService merchantService;
    private final MerchantRelationService merchantRelationService;
    private final OmsWithdrawService withdrawService;

    @Override
    public Asset queryAsset(Long userId, Integer type) {
        LambdaQueryWrapper<Asset> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Asset::getUserId, userId);
        wrapper.eq(Asset::getType, type);
        Asset asset = baseMapper.selectOne(wrapper);
        if (asset != null) {
            return asset;
        }
        asset = new Asset();
        asset.setFrozen(new BigDecimal("0"));
        asset.setType(type);
        asset.setTotal(new BigDecimal("0"));
        asset.setUpdateTime(LocalDateTime.now());
        asset.setUserId(userId);
        baseMapper.insert(asset);
        return asset;
    }

    @Override
    public BigDecimal updateAsset(Integer operateType, BigDecimal operateAmount, Long userId, Integer type, Integer amountType) {
        Asset asset = queryAsset(userId, type);
        if(amountType == 1){
            BigDecimal total = asset.getTotal();
            if (operateType == 1 && total.compareTo(operateAmount) < 0) {
                throw new ResourceUnavailableException("账户余额不足");
            }
            if (operateType == 0) {
                asset.setTotal(total.add(operateAmount));
            } else if (operateType == 1) {
                asset.setTotal(total.subtract(operateAmount));
            }
        } else if(amountType == 2){
            BigDecimal frozen = asset.getFrozen();
            if (operateType == 1 && frozen.compareTo(operateAmount) < 0) {
                throw new ResourceUnavailableException("账户冻结余额不足");
            }
            if (operateType == 0) {
                asset.setFrozen(frozen.add(operateAmount));
            } else if (operateType == 1) {
                asset.setFrozen(frozen.subtract(operateAmount));
            }
        }
        asset.setUpdateTime(LocalDateTime.now());
        int i = baseMapper.updateById(asset);
        if (i != 1) {
            throw new ResourceUnavailableException("操作失败");
        }
        return asset.getTotal();
    }

    @Override
    public void updatePlatformAsset(BigDecimal platformRealityAmount, BigDecimal payAmount) {
        Asset asset = queryAsset(PlatformConstant.PLATFORM_ID, 0);
        BigDecimal frozen = asset.getFrozen();
        if(frozen.compareTo(payAmount) < 0){
            throw new ResourceUnavailableException("账户冻结余额不足");
        }
        baseMapper.updatePlatformAsset(platformRealityAmount, payAmount, LocalDateTime.now(), PlatformConstant.PLATFORM_ID);
    }

    @Override
    public MerchantAssetDTO getMerchantAsset(Long merchantId) {
        if(merchantId == null){
            return null;
        }
        MerchantAssetDTO dto = new MerchantAssetDTO();
        WalletDTO walletDTO = balanceDetailService.getMerchantAsset(merchantId);
        LambdaQueryWrapper<Asset> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(Asset::getUserId, merchantId);
        wrapper.eq(Asset::getType, 2);
        Asset asset = baseMapper.selectOne(wrapper);
        if(asset != null){
            BigDecimal total = asset.getTotal();
            BigDecimal frozen = asset.getFrozen();
            walletDTO.setAsset(total);
            walletDTO.setWithdraw(total.subtract(frozen));
        }
        dto.setWalletDTO(walletDTO);
        OrderDTO orderDTO = orderService.queryOrderManage(merchantId);
        dto.setOrderDTO(orderDTO);
        MerchantTeamDTO merchantTeamDTO = orderService.queryTeamOrder(merchantId);
        dto.setMerchantTeamDTO(merchantTeamDTO);
        return dto;
    }

    @Override
    public void withdraw(WithdrawParam param) {
        Integer type = param.getType();
        BigDecimal amount = param.getAmount();
        Long userId = SessionContextHolder.getId();
        if(type == null || (type != 1 && type != 2)){
            throw new ResourceUnavailableException("提现类型错误");
        }
        if(type == 2){
            //商家体现
            Long merchantId = param.getMerchantId();
            MerchantDTO merchantDTO = merchantService.queryCheck(merchantId, userId);
            if(merchantDTO == null){
                throw new ResourceUnavailableException("该账户下商家不存在");
            }
            userId = merchantId;
        }
        Asset asset = queryAsset(userId, type);
        BigDecimal total = asset.getTotal();
        if(amount.compareTo(total) > 0){
            throw new ResourceUnavailableException("账户余额不足");
        }
        //查询首信易商户号
        MerchantRelation merchantRelation = merchantRelationService.queryByUserIdAndType(userId, type);
        if(merchantRelation == null){
            throw new ResourceUnavailableException("请联系管理员开通商户号");
        }
        //生成提现订单
        Long withdrawId = withdrawService.createWithdraw(userId, type, amount, 1);

        String sxyMerchantId = merchantRelation.getSxyMerchantId();
        //生成提现参数
        HashMap<String, Object> map = new HashMap<>();
        map.put("merchantId", sxyMerchantId);
        map.put("requestId", withdrawId);
        map.put("withdrawAmount", amount.multiply(new BigDecimal("100")));
        map.put("notifyUrl", TradeConstant.WITHDRAW_NOTIFY_URL);
        map.put("remark", "");
        Gson gson = new Gson();
        TradeHandle.handle(gson.toJson(map), TradeConstant.WITHDRAW_URL, sxyMerchantId, TradeConstant.PARTNER_ID);
    }
}
